@@ -38,6 +38,7 @@
 #define DEFAULT_ADAPTER			"DefaultAdapter"
 #define ADAPTER_REMOVED			"AdapterRemoved"
 #define DEFAULT_ADAPTER_CHANGED		"DefaultAdapterChanged"
+#define ADAPTER_PROPERTY_CHANGED	"PropertyChanged"
 #define MANAGER_PATH			"/"
 #define OOB_AGENT			"/org/neard/agent/neard_oob"
 
@@ -94,6 +95,7 @@ static struct near_oob_data bt_def_oob_data;
 static guint watch;
 static guint removed_watch;
 static guint adapter_watch;
+static guint adapter_props_watch;
 
 static guint register_bluez_timer;
 
@@ -155,7 +157,7 @@ static int bt_generic_call(DBusConnection *conn,
 
 	if (msg == NULL) {
 		near_error("Unable to allocate new D-Bus %s message", method);
-		err = -ENOMEM;
+		return -ENOMEM;
 	}
 
 	va_start(args, type);
@@ -415,6 +417,66 @@ fail:
 	return -ENOMEM;
 }
 
+static gboolean bt_adapter_property_changed(DBusConnection *conn,
+							DBusMessage *message,
+							void *user_data)
+{
+	DBusMessageIter iter;
+	DBusMessageIter var;
+	const char *property;
+
+	if (dbus_message_iter_init(message, &iter) == FALSE)
+		return TRUE;
+
+	dbus_message_iter_get_basic(&iter, &property);
+	dbus_message_iter_next(&iter);
+
+	if (dbus_message_iter_get_arg_type(&iter) != DBUS_TYPE_VARIANT)
+		return TRUE;
+
+	dbus_message_iter_recurse(&iter, &var);
+
+	if (g_str_equal(property, "Name") == TRUE) {
+		const char *name;
+
+		if (dbus_message_iter_get_arg_type(&var) != DBUS_TYPE_STRING)
+			return TRUE;
+
+		dbus_message_iter_get_basic(&var, &name);
+
+		g_free(bt_def_oob_data.bt_name);
+		bt_def_oob_data.bt_name = g_strdup(name);
+
+		if (bt_def_oob_data.bt_name != NULL)
+			bt_def_oob_data.bt_name_len = strlen(name);
+		else
+			bt_def_oob_data.bt_name_len = 0;
+
+		DBG("%s: %s", property, name);
+	} else if (g_str_equal(property, "Class") == TRUE) {
+		int class;
+
+		if (dbus_message_iter_get_arg_type(&var) != DBUS_TYPE_UINT32)
+			return TRUE;
+
+		dbus_message_iter_get_basic(&var, &class);
+		bt_def_oob_data.class_of_device = class;
+
+		DBG("%s: %x", property, bt_def_oob_data.class_of_device);
+	} else if (g_str_equal(property, "Powered") == TRUE) {
+		dbus_bool_t powered;
+
+		if (dbus_message_iter_get_arg_type(&var) != DBUS_TYPE_BOOLEAN)
+			return TRUE;
+
+		dbus_message_iter_get_basic(&var, &powered);
+		bt_def_oob_data.powered = powered;
+
+		DBG("%s: %u", property, bt_def_oob_data.powered);
+	}
+
+	return TRUE;
+}
 
 /* Get default local adapter properties */
 static void bt_get_properties_cb(DBusPendingCall *pending, void *user_data)
@@ -441,6 +503,12 @@ static void bt_get_properties_cb(DBusPendingCall *pending, void *user_data)
 	else
 		DBG("Get Properties complete: %s", bt_props->def_adapter);
 
+	adapter_props_watch = g_dbus_add_signal_watch(bt_conn, NULL, NULL,
+						ADAPTER_INTF,
+						ADAPTER_PROPERTY_CHANGED,
+						bt_adapter_property_changed,
+						NULL, NULL);
+
 	/* clean */
 	dbus_message_unref(reply);
 	dbus_pending_call_unref(pending);
@@ -610,7 +678,7 @@ static void bt_parse_eir(uint8_t *eir_data, uint16_t eir_data_len,
  * mime_properties is a bitmask and should reflect the fields found in
  * the incoming oob.
  */
-int __near_bluetooth_parse_oob_record(struct bt_data *data,
+int __near_bluetooth_parse_oob_record(struct carrier_data *data,
 						uint16_t *mime_properties,
 						near_bool_t pair)
 {
@@ -688,11 +756,14 @@ int __near_bluetooth_parse_oob_record(struct bt_data *data,
 		}
 		ptr = ptr + oob->bt_name_len;
 	} else {
+		bt_eir_free(oob);
 		return -EINVAL;
 	}
 
-	if (pair == FALSE)
+	if (pair == FALSE) {
+		bt_eir_free(oob);
 		return 0;
+	}
 
 	/* check and get the default adapter */
 	oob->def_adapter = g_strdup(bt_def_oob_data.def_adapter);
@@ -778,9 +849,9 @@ done:
  * mime_props is a bitmask we use to add or not specific fields in the
  * oob frame (e.g.: OOB keys)
  * */
-struct bt_data *__near_bluetooth_local_get_properties(uint16_t mime_props)
+struct carrier_data *__near_bluetooth_local_get_properties(uint16_t mime_props)
 {
-	struct bt_data *data = NULL;
+	struct carrier_data *data = NULL;
 	uint8_t offset;
 
 	char hash[OOB_SP_SIZE];
@@ -867,6 +938,11 @@ struct bt_data *__near_bluetooth_local_get_properties(uint16_t mime_props)
 
 	data->data[0] = data->size ;
 
+	if (bt_def_oob_data.powered == TRUE)
+		data->state = CPS_ACTIVE;
+	else
+		data->state = CPS_INACTIVE;
+
 	return data;
 
 fail:
@@ -887,6 +963,9 @@ static gboolean bt_adapter_removed(DBusConnection *conn, DBusMessage *message,
 	if (bt_props->def_adapter == NULL)
 		return TRUE;
 
+	g_dbus_remove_watch(bt_conn, adapter_props_watch);
+	adapter_props_watch = 0;
+
 	if (dbus_message_iter_init(message, &iter) == FALSE)
 		return TRUE;
 
@@ -916,6 +995,9 @@ static gboolean bt_default_adapter_changed(DBusConnection *conn,
 	if (dbus_message_iter_init(message, &iter) == FALSE)
 		return TRUE;
 
+	g_dbus_remove_watch(bt_conn, adapter_props_watch);
+	adapter_props_watch = 0;
+
 	dbus_message_iter_get_basic(&iter, &adapter_path);
 	DBG("New default adapter [%s]", adapter_path);
 
@@ -972,7 +1054,7 @@ static void bt_connect(DBusConnection *conn, void *data)
 {
 	DBG("connection %p with %p", conn, data);
 
-	if (__near_agent_handover_registered() == TRUE) {
+	if (__near_agent_handover_registered(HO_AGENT_BT) == TRUE) {
 		DBG("Agent already registered");
 		return;
 	}
@@ -1003,11 +1085,14 @@ static void bt_disconnect(DBusConnection *conn, void *user_data)
 
 	g_dbus_remove_watch(bt_conn, adapter_watch);
 	adapter_watch = 0;
+
+	g_dbus_remove_watch(bt_conn, adapter_props_watch);
+	adapter_props_watch = 0;
 }
 
 static int bt_prepare_handlers(DBusConnection *conn)
 {
-	if (__near_agent_handover_registered() == TRUE)
+	if (__near_agent_handover_registered(HO_AGENT_BT) == TRUE)
 		return 0;
 
 	watch = g_dbus_add_service_watch(bt_conn, BLUEZ_SERVICE,
