@@ -26,11 +26,40 @@
 #define AF_NFC 39
 #endif
 
+struct sniffer_packet {
+	guint8 adapter_idx;
+	guint8 direction;
+
+	struct {
+		guint8 ptype;
+		guint8 local_sap;
+		guint8 remote_sap;
+
+		guint8 send_seq;
+		guint8 recv_seq;
+
+		guint8 *data;
+		guint32 data_len;
+	} llcp;
+
+	struct {
+		guint8 version;
+
+		guint8 rcode;
+
+		guint32 acceptable_len;
+
+		guint8 *data;
+		guint32 data_len;
+		guint32 real_len;
+	} snep;
+};
+
 int sniffer_init(void);
 
 void sniffer_cleanup(void);
 
-void sniffer_print_hexdump(FILE *file, unsigned char *data, int len,
-							char *line_prefix);
+void sniffer_print_hexdump(FILE *file, guint8 *data, guint32 len,
+			   guint8 indent, gboolean print_len);
 
 #endif /* __SNIFFER_H */
